// Error codes for the ApogeeNet.cpp file


#define APN_NET_SUCCESS						0

#define APN_NET_ERR_CONNECT					1
#define APN_NET_ERR_CLOSE					2

#define APN_NET_ERR_HTTP_SESSION			3
#define APN_NET_ERR_HTTP_OPEN_REQUEST		4
#define APN_NET_ERR_HTTP_SEND_REQUEST		5
#define APN_NET_ERR_GENERIC_CGI				6

#define APN_NET_ERR_IMAGE_PARAMS			7
#define APN_NET_ERR_IMAGE_DATA				8
#define APN_NET_ERR_IMAGE_START				9
#define APN_NET_ERR_IMAGE_EXPOSING			10

#define APN_NET_ERR_UPLOAD_FILE				11

#define APN_NET_ERR_SERIAL_CONNECT			12
#define APN_NET_ERR_SERIAL_ID				13
#define APN_NET_ERR_SERIAL_BAUDRATE			14
#define APN_NET_ERR_SERIAL_PARITY			15
#define APN_NET_ERR_SERIAL_READ_INPUT		16
#define APN_NET_ERR_SERIAL_WRITE_INPUT		17
#define APN_NET_ERR_SERIAL_NO_CONNECTION	18
#define APN_NET_ERR_SERIAL_READ_FAILURE		19
#define APN_NET_ERR_SERIAL_WRITE_FAILURE	20


