// Error codes for the ApogeeUsb* files


#define APN_USB_SUCCESS						0

#define APN_USB_ERR_OPEN					1
#define APN_USB_ERR_READ					2
#define APN_USB_ERR_WRITE					3

#define APN_USB_ERR_IMAGE_DOWNLOAD			4

#define APN_USB_ERR_START_EXP				5
#define APN_USB_ERR_STOP_EXP				6

#define APN_USB_ERR_STATUS					7

#define APN_USB_ERR_RESET					8

#define APN_USB_ERR_VENDOR_INFO				9

#define APN_USB_ERR_DRIVER_VERSION			10

#define APN_USB_ERR_CUSTOM_SN_READ			11
#define APN_USB_ERR_CUSTOM_SN_WRITE			12

#define APN_USB_ERR_8051_REV_READ			13

#define APN_USB_ERR_START_CI				14
#define APN_USB_ERR_STOP_CI					15

#define APN_USB_ERR_SERIAL_ID				20
#define APN_USB_ERR_SERIAL_OPEN				21
#define APN_USB_ERR_SERIAL_READ_SETTINGS	22
#define APN_USB_ERR_SERIAL_WRITE_SETTINGS	23
#define APN_USB_ERR_SERIAL_READ_PORT		24
#define APN_USB_ERR_SERIAL_WRITE_PORT		25

#define APN_USB_FAIL_DEVNAME_COMPARE		100


